import cv2
import matplotlib as plt
import os
import random
import shutil
from PIL import Image, ImageOps


def histogram():

    # Load the image
    img = cv2.imread('image.jpg')
    # Plot the histogram before normalization
    plt.figure()
    plt.title('Before Normalization')
    plt.hist(img.ravel(), bins=256, range=(0, 255), fc='k', ec='k')

    # Normalize the image
    img = cv2.normalize(img, None, 0, 255, cv2.NORM_MINMAX)

    # Plot the histogram after normalization
    plt.figure()
    plt.title('After Normalization')
    plt.hist(img.ravel(), bins=256, range=(0, 255), fc='k', ec='k')

    # Show the plots
    plt.show()


def flip(dirs):
    for item in dirs:
        image = Image.open(input_path + item)
        image.save(target_path + item, 'JPEG', quality=95, subsampling=0)
        image_flip = ImageOps.flip(image)
        image_flip.save(target_path + item[:-4] + '_flip.jpg', 'JPEG', quality=95, subsampling=0)


def resize(dirs):
    for item in dirs:
        image = Image.open(input_path + item)
        image.save(target_path + item[:-4] + '.jpg', 'JPEG', quality=95, subsampling=0)


def greyscale(dirs):
    for item in dirs:
        image = Image.open(input_path + item)
        im_greyscale = image.resize((128, 128), Image.ANTIALIAS)
        im_greyscale = im_greyscale.convert(mode='L')
        im_greyscale.save(target_path + item[:-4] + '.jpg', 'JPEG', quality=95, subsampling=0)


def rotate(dirs, degrees):
    for item in dirs:
        image = Image.open(input_path + item)
        im_resize = image.rotate(degrees)
        im_resize.save(target_path + item[:-4] + str(degrees) + '_rot.jpg', 'JPEG', quality=95, subsampling=0)


def zoom_at(img, x, y, zoom):
    w, h = img.size
    zoom2 = zoom * 2
    img = img.crop((x - w / zoom2, y - h / zoom2,
                    x + w / zoom2, y + h / zoom2))
    return img.resize((w, h), Image.LANCZOS)


def move_files(src_folder, dst_folder):

    files = os.listdir(src_folder)
    # Calculate the number of files to move (50% of the total number of files)
    num_files_to_move = int(len(files) * 0.5)

    # Randomly select the files to move
    files_to_move = random.sample(files, num_files_to_move)

    # Move the selected files to the destination folder
    for file in files_to_move:
        src = os.path.join(src_folder, file)
        dst = os.path.join(dst_folder, file)
        shutil.move(src, dst)


def clear_folder(dirs):
    for file in os.listdir(dirs):
        os.remove(target_path + file)


input_path = "<source_folder_path>"
target_path = "<target_folder_path>"

directory = os.listdir(input_path)
