import tensorflow as tf


def custom_model(img_height, img_width):
    model = tf.keras.Sequential([
        tf.keras.layers.Rescaling(1. / 255, input_shape=(img_height, img_width, 3)),
        tf.keras.layers.Conv2D(16, 3, padding='same', activation='relu'),
        tf.keras.layers.MaxPooling2D(pool_size=2, strides=2, padding='same'),
        tf.keras.layers.Conv2D(32, 3, padding='same', activation='relu'),
        tf.keras.layers.Conv2D(32, 3, padding='same', activation='relu'),
        tf.keras.layers.MaxPooling2D(pool_size=2, strides=2, padding='same'),
        tf.keras.layers.Conv2D(64, 3, padding='same', activation='relu'),
        tf.keras.layers.Conv2D(64, 3, padding='same', activation='relu'),
        tf.keras.layers.MaxPooling2D(pool_size=2, strides=2, padding='same'),
        tf.keras.layers.Dropout(0.2),
        tf.keras.layers.Flatten(),
        tf.keras.layers.Dense(128, activation='relu'),
        tf.keras.layers.Dense(2, 'softmax')
    ])
    return model


def MobileNetV2(img_height, img_width):
    mob_model = tf.keras.applications.MobileNetV2(weights='imagenet', include_top=False,
                                                  input_shape=(img_height, img_width, 3))

    # Freeze the layers of the base_model
    mob_model.trainable = False

    x = mob_model.output
    x = tf.keras.layers.GlobalAveragePooling2D()(x)
    x = tf.keras.layers.Dense(128, activation='relu')(x)
    x = tf.keras.layers.Flatten()(x)
    x = tf.keras.layers.Dropout(0.2)(x)
    predictions = tf.keras.layers.Dense(2, activation='softmax')(x)

    # Create the full model
    model = tf.keras.Model(inputs=mob_model.input, outputs=predictions)
    return model


def resnet_50(img_height, img_width):
    # Load the ResNet-50 model pre-trained on ImageNet
    res_model = tf.keras.applications.ResNet50(weights='imagenet', include_top=False,
                                               input_shape=(img_height, img_width, 3))

    # Freeze the layers of the base_model so the weights do not get updated
    res_model.trainable = False

    x = res_model.output
    x = tf.keras.layers.GlobalAveragePooling2D()(x)
    x = tf.keras.layers.Dense(128, activation='relu')(x)
    x = tf.keras.layers.Flatten()(x)
    x = tf.keras.layers.Dropout(0.2)(x)
    predictions = tf.keras.layers.Dense(2, activation='softmax')(x)
    # Create the full model
    model = tf.keras.Model(inputs=res_model.input, outputs=predictions)

    return model
