import matplotlib.pyplot as plt
import tensorflow as tf
from sklearn.metrics import confusion_matrix, classification_report
import seaborn as sns
import numpy as np


def report(test_ds, ml_model):
    sns.set_style('darkgrid')
    classes = test_ds.class_names
    ytrue = []
    for images, label in test_ds:
        for e in label:
            ytrue.append(classes[e])
    ypredicted = []
    errors = 0
    count = 0
    predictions = ml_model.predict(test_ds, verbose=1)  # predict on the test data
    for i, p in enumerate(predictions):
        count += 1
        index = np.argmax(p)
        print(index)
        pred_class = classes[index]
        ypredicted.append(pred_class)
        if pred_class != ytrue[i]:
            print(pred_class, ytrue[i])
            errors += 1
    acc = (count - errors) * 100 / count
    msg = f'there were {count - errors} correct predictions in {count} tests for an accuracy of {acc:6.2f} % '
    print(msg)
    ypred = np.array(ypredicted)
    ytrue = np.array(ytrue)
    cm = confusion_matrix(ytrue, ypred)
    length = len(classes)
    fig_width = 8
    fig_height = 8
    plt.figure(figsize=(fig_width, fig_height))
    sns.heatmap(cm, annot=True, vmin=0, fmt='g', cmap='Blues', cbar=False)
    plt.xticks(np.arange(length) + .5, classes, rotation=90)
    plt.yticks(np.arange(length) + .5, classes, rotation=0)
    plt.xlabel("Predicted")
    plt.ylabel("Actual")
    plt.title("Confusion Matrix")
    plt.show()
    clr = classification_report(ytrue, ypred, target_names=classes)
    print("Classification Report:\n----------------------\n", clr)


batch_size = 32
img_height = 256
img_width = 256

input_path = "<Validation_Path>"

valid_ds = tf.keras.utils.image_dataset_from_directory(
    input_path2,
    color_mode="rgb",
    seed=2023,
    shuffle=False,
    image_size=(img_height, img_width),
    batch_size=batch_size)

model = tf.keras.models.load_model('Proposed_CNN.h5')
test_loss, test_acc = model.evaluate(valid_ds, verbose=0)

report(valid_ds, model)
