import matplotlib.pyplot as plt
import tensorflow as tf
import Model_architectures as Arch


batch_size = 32
img_height = 128
img_width = 128

trainset_path = "<Trainset_Path>"
validation_path = "<Validation_Path>"

train_ds = tf.keras.utils.image_dataset_from_directory(
    trainset_path,
    labels="inferred",
    color_mode="rgb",
    seed=2023,
    image_size=(img_height, img_width),
    batch_size=batch_size)

class_names = train_ds.class_names
classes_num = len(class_names)
print(class_names)

valid_ds = tf.keras.utils.image_dataset_from_directory(
    validation_path,
    labels="inferred",
    color_mode="rgb",
    seed=2023,
    image_size=(img_height, img_width),
    batch_size=batch_size)

for image_batch, labels_batch in train_ds:
    print(image_batch.shape)
    print(labels_batch.shape)
    break

auto_tune = tf.data.AUTOTUNE

train_ds = train_ds.cache().shuffle(1000).prefetch(buffer_size=auto_tune)
valid_ds_tuned = valid_ds.cache().prefetch(buffer_size=auto_tune)

model = Arch.MobileNetV2(img_height, img_width)

model.compile(tf.keras.optimizers.RMSprop(learning_rate=0.0001),
              loss=tf.keras.losses.SparseCategoricalCrossentropy(),
              metrics=['accuracy'])


model.summary()

epochs = 100

training_history = model.fit(
    train_ds,
    validation_data=valid_ds_tuned,
    epochs=epochs,
)

model.save("Proposed_CNN.h5")

accuracy = training_history.history['accuracy']
validation_accuracy = training_history.history['val_accuracy']

training_loss = training_history.history['loss']
validation_loss = training_history.history['val_loss']

epochs_range = range(epochs)

plt.figure(figsize=(8, 8))
plt.plot(epochs_range, accuracy, label='Training Accuracy', color="red")
plt.plot(epochs_range, validation_accuracy, label='Validation Accuracy', color="blue")
plt.axhline(y=max(validation_accuracy), color='gray', linestyle='--')
plt.legend(loc='lower right')
plt.title('Training and Validation Accuracy')
plt.xlabel('Epochs')
plt.ylabel('Accuracy')

plt.savefig('accuracy.png')
plt.show()

plt.plot(epochs_range, training_loss, label='Training Loss', color="red")
plt.plot(epochs_range, validation_loss, label='Validation Loss', color="blue")
plt.legend(loc='lower left')
plt.title('Training and Validation Loss')
plt.xlabel('Epochs')
plt.ylabel('Loss')
plt.savefig('loss.png')
plt.show()
